/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
package arcgissamples.soe;

import java.util.logging.Logger;

import com.esri.arcgis.carto.AlternatingScaleBar;
import com.esri.arcgis.carto.IActiveView;
import com.esri.arcgis.carto.IElement;
import com.esri.arcgis.carto.IFrameElement;
import com.esri.arcgis.carto.IGraphicsContainer;
import com.esri.arcgis.carto.IImageDescription;
import com.esri.arcgis.carto.IImageDisplay;
import com.esri.arcgis.carto.IImageType;
import com.esri.arcgis.carto.ILayoutImage;
import com.esri.arcgis.carto.ILegend;
import com.esri.arcgis.carto.IMap;
import com.esri.arcgis.carto.IMapFrame;
import com.esri.arcgis.carto.IMapServerLayout;
import com.esri.arcgis.carto.IMapServerObjects;
import com.esri.arcgis.carto.IMapSurroundFrame;
import com.esri.arcgis.carto.ImageDescription;
import com.esri.arcgis.carto.ImageDisplay;
import com.esri.arcgis.carto.ImageType;
import com.esri.arcgis.carto.Legend;
import com.esri.arcgis.carto.MapServer;
import com.esri.arcgis.carto.MarkerNorthArrow;
import com.esri.arcgis.carto.PageLayout;
import com.esri.arcgis.carto.TextElement;
import com.esri.arcgis.carto.esriImageFormat;
import com.esri.arcgis.carto.esriImageReturnType;
import com.esri.arcgis.carto.esriViewDrawPhase;
import com.esri.arcgis.geometry.Envelope;
import com.esri.arcgis.geometry.IEnvelope;
import com.esri.arcgis.geometry.IGeometry;
import com.esri.arcgis.geometry.IPoint;
import com.esri.arcgis.geometry.Point;
import com.esri.arcgis.system.IUID;
import com.esri.arcgis.system.UID;

public class PageLayoutImpl
{
	private MapServer serverObject;

	// SOE metadata
	private int width;
	private int height;
	private int dpi;
	
	//Document related metadata
	private String mapName;
	private IMap map;
	private IActiveView activeView;
	private IActiveView pageActiveView; 
	private IMapServerObjects serverObjects;
	private PageLayout pageLayout;	

	// SOE result
	private String outputUrl;
	private static final Logger logger = Logger.getLogger(PageLayoutImpl.class.getName());

	// public constructor.
	public PageLayoutImpl(MapServer serverObject, int width, int height, int dpi)
	{
		this.serverObject = serverObject;
		this.width = width;
		this.height = height;
		this.dpi = dpi;
		this.outputUrl = null;
	}
	
	  public void generatePageLayout() throws Exception {
		    try {
		    	
		      mapName = serverObject.getDefaultMapName();
		      
		      map = serverObject.getMap(mapName);
		      
		      activeView = coerce(map);
		      
		      serverObjects = coerce(serverObject);
		      pageLayout = coerce(serverObjects.getPageLayout());
		      
		      pageActiveView = coerce(pageLayout);
		      
		      IGraphicsContainer graphicsContainer = coerce(pageLayout);
		      
		      graphicsContainer.reset();     
		      IFrameElement frameElement = graphicsContainer.findFrame(map);
		      IMapFrame mapFrame = coerce(frameElement);
		      
		      //create a legend
		      IUID uid = coerce(new UID());
		      uid.setValue(Legend.getClsid());      
		      
		      IMapSurroundFrame legendFrame = mapFrame.createSurroundFrame(uid, null);
		      legendFrame.getMapSurround().setName("Map Legend");
		      
		      IElement legendElement = coerce(legendFrame);
		      legendElement.activate(pageActiveView.getScreenDisplay());
		      
		      ILegend legend = coerce(legendFrame.getMapSurround());
		      
		      legend.setTitle("Map Legend.");
		      legend.setAutoAdd(true);
		      legend.refresh();      
		      
		      IElement mapElement = coerce(mapFrame);			      
		      IEnvelope mainEnvelope = mapElement.getGeometry().getEnvelope();
		      IEnvelope elementEnvelope = coerce(new Envelope());
		      
		      double xmin = mainEnvelope.getXMax() + 2;
		      double ymin = mainEnvelope.getYMin() + 0.5;
		      double xmax = mainEnvelope.getXMax() - 2;
		      double ymax = mainEnvelope.getYMax() - 0.5;
		      elementEnvelope.putCoords(xmin, ymin, xmax, ymax);
		      
		      legendElement.setGeometry((IGeometry)elementEnvelope);
		      
		      //add it to the graphicsContainer.
		      graphicsContainer.addElement(legendElement, 0);
		      
		      //add a North Arrow
		      IUID uidNorth = coerce(new UID());
		      uidNorth.setValue(MarkerNorthArrow.getClsid());
		      
		      IEnvelope northArrowEnvelope = coerce(new Envelope());
		      northArrowEnvelope.putCoords(xmin , ymax - 0.5, xmin + 2, ymax + 1);
		      
		      IMapSurroundFrame northArrowFrame = mapFrame.createSurroundFrame(uidNorth, null);
		            
		      IElement northElement = coerce(northArrowFrame);
		      northElement.setGeometry(northArrowEnvelope);
		      
		      graphicsContainer.addElement(northElement, 0);
		      
		      //add a Title
		      TextElement title = new TextElement();
		      title.setSize(20);
		      title.setText(serverObject.getConfigurationName().toUpperCase());
		                  
		      IPoint titleGeometry = coerce(new Point());
		      
		      titleGeometry.setX((ymax - ymin) / 2);
		      titleGeometry.setY(((xmin + xmax) / 2) + 2);
		      
		      IElement titleElement = coerce(title);
		      titleElement.setGeometry(titleGeometry);
		      
		      //title element is always on top
		      graphicsContainer.addElement(titleElement, 1);
		      
		      //Add a ScaleBar
		      IUID uidScaleBar = coerce(new UID());
		      uidScaleBar.setValue(AlternatingScaleBar.getClsid());
		      
		      IMapSurroundFrame scaleBarFrame = mapFrame.createSurroundFrame(uidScaleBar, null);
		      
		      IEnvelope scaleEnvelope = coerce(new Envelope());
		      scaleEnvelope.putCoords(1, 1, 3, 1.3);
		      
		      IElement scaleElement = coerce(scaleBarFrame);
		      scaleElement.setGeometry(scaleEnvelope);
		      
		      //Scale bar on top.
		      graphicsContainer.addElement(scaleElement, 1);
		      graphicsContainer.reset();
		      
		      activeView.partialRefresh(esriViewDrawPhase.esriViewGraphics, null, null);
		      pageActiveView.partialRefresh(esriViewDrawPhase.esriViewGraphics, null, null);
		      
		      //Generate Layout
		      IMapServerLayout mapLayout = coerce(serverObject);
		      IImageDescription imageDesc = coerce(new ImageDescription());
		      
		      IImageDisplay imageDisplay = coerce(new ImageDisplay());
		      imageDisplay.setDeviceResolution(dpi);
		      imageDisplay.setWidth(width);
		      imageDisplay.setHeight(height);
		      
		      IImageType iImageType = coerce(new ImageType());
		      iImageType.setFormat(esriImageFormat.esriImagePNG24);
		      iImageType.setReturnType(esriImageReturnType.esriImageReturnURL);
		      
		      imageDesc.setDisplay(imageDisplay);
		      imageDesc.setType(iImageType);
		      
		      ILayoutImage outputImage = mapLayout.exportLayout(mapLayout.getDefaultPageDescription(), imageDesc);
		      
		      //dont persist -> restore back to original state.
		      graphicsContainer.deleteElement(legendElement);		      
		      graphicsContainer.deleteElement(northElement);		      
		      graphicsContainer.deleteElement(titleElement);
		      graphicsContainer.deleteElement(scaleElement);
		      
		      graphicsContainer.reset();
		      
		      outputUrl = outputImage.getURL();
		      logger.info("Generated Layout : " + outputImage.getURL());
		            
		    }catch (Exception e) {
		      logger.severe("Error generating page layout : " + e.getMessage());
		      throw e;
		    }
		  }	

	/*public void generatePageLayout()
	{
		try
		{
			Map map = (Map) serverObject.getMap(serverObject.getDefaultMapName());

			PageLayout pageLayout = (PageLayout) map.getPageLayout();
			pageLayout.reset();

			MapFrame mapFrame = (MapFrame) pageLayout.findFrame(map);

			// create a legend
			UID uid = new UID();
			uid.setValue(Legend.getClsid());

			MapSurroundFrame legendFrame = (MapSurroundFrame) mapFrame.createSurroundFrame(uid, null);
			legendFrame.getMapSurround().setName("Map Legend");
			legendFrame.activate(pageLayout.getScreenDisplay());

			Legend legend = (Legend) legendFrame.getMapSurround();
			legend.setTitle("Map Legend.");
			legend.setAutoAdd(true);
			legend.refresh();
			
			legendFrame.setMapSurroundByRef(legend);//added by Ajit

			Envelope mainEnvelope = (Envelope) mapFrame.getGeometry().getEnvelope();
			Envelope elementEnvelope = new Envelope();

			double xmin = mainEnvelope.getXMax() + 2;
			double ymin = mainEnvelope.getYMin() + 0.5;
			double xmax = mainEnvelope.getXMax() - 2;
			double ymax = mainEnvelope.getYMax() - 0.5;
			elementEnvelope.putCoords(xmin, ymin, xmax, ymax);

			legendFrame.setGeometry(elementEnvelope);

			// add it to the pagelayout.
			pageLayout.addElement(legendFrame, 0);

			// add a North Arrow
			UID uidNorth = new UID();
			uidNorth.setValue(MarkerNorthArrow.getClsid());

			Envelope northArrowEnvelope = new Envelope();
			northArrowEnvelope.putCoords(xmin, ymax - 0.5, xmin + 2, ymax + 1);

			MapSurroundFrame northArrowFrame = (MapSurroundFrame) mapFrame.createSurroundFrame(uidNorth, null);
			northArrowFrame.setGeometry(northArrowEnvelope);

			pageLayout.addElement(northArrowFrame, 0);

			// add a Title
			TextElement title = new TextElement();
			title.setSize(20);
			title.setText("USA Map Service.");

			Point titleGeometry = new Point();
			titleGeometry.setX((ymax - ymin) / 2);
			titleGeometry.setY(((xmin + xmax) / 2) + 2);

			title.setGeometry(titleGeometry);

			// title element is always on top
			pageLayout.addElement(title, 1);

			// Add a ScaleBar
			UID uidScaleBar = new UID();
			uidScaleBar.setValue(AlternatingScaleBar.getClsid());

			MapSurroundFrame scaleBarFrame = (MapSurroundFrame) mapFrame.createSurroundFrame(uidScaleBar, null);

			Envelope scaleEnvelope = new Envelope();
			scaleEnvelope.putCoords(1, 1, 3, 1.3);

			scaleBarFrame.setGeometry(scaleEnvelope);

			// Scale bar on top.
			pageLayout.addElement(scaleBarFrame, 1);
			pageLayout.reset();

			map.partialRefresh(esriViewDrawPhase.esriViewGraphics, null, null);
			pageLayout.partialRefresh(esriViewDrawPhase.esriViewGraphics, null, null);

			// Generate Layout
			// IMapServerLayout mapLayout = Macros.coerce(serverObject);
			ImageDescription imageDesc = new ImageDescription();

			ImageDisplay imageDisplay = new ImageDisplay();
			imageDisplay.setDeviceResolution(dpi);
			imageDisplay.setWidth(width);
			imageDisplay.setHeight(height);

			ImageType iImageType = new ImageType();
			iImageType.setFormat(esriImageFormat.esriImagePNG24);
			iImageType.setReturnType(esriImageReturnType.esriImageReturnURL);

			imageDesc.setDisplay(imageDisplay);
			imageDesc.setType(iImageType);

			ILayoutImage outputImage = serverObject.exportLayout(serverObject.getDefaultPageDescription(), imageDesc);

			// dont persist -> restore back to original state.
			pageLayout.deleteElement(legendFrame);
			pageLayout.deleteElement(northArrowFrame);
			pageLayout.deleteElement(title);
			pageLayout.deleteElement(scaleBarFrame);

			pageLayout.reset();

			outputUrl = outputImage.getURL();
			logger.info("Generated Layout : " + outputImage.getURL());
		}
		catch (AutomationException e)
		{
			logger.info("MapLayoutImpl exception: " + e.getMessage() + " - " + e.getDescription());
			e.printStackTrace();
		}
		catch (UnknownHostException e)
		{
			logger.info("MapLayoutImpl exception: " + e.getMessage() + " - " + e.getLocalizedMessage());
			e.printStackTrace();
		}
		catch (IOException e)
		{
			logger.info("MapLayoutImpl exception: " + e.getMessage() + " - " + e.getLocalizedMessage());
			e.printStackTrace();
		}

	}*/

	public String getOutputUrl()
	{
		return outputUrl;
	}
	
	// This method is a convinience method to avoid all those interface type casts.
	@SuppressWarnings("unchecked")
	private <F, T> T coerce(F from)
	{
		return (T) from;
	}

	@SuppressWarnings("unused")
	private boolean isEmpty(String str)
	{
		if (str == null || str.length() == 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}
