/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
package arcgissamples.soe;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import com.esri.arcgis.carto.MapServer;
import com.esri.arcgis.interop.AutomationException;
import com.esri.arcgis.interop.extn.ArcGISExtension;
import com.esri.arcgis.server.IServerObjectExtension;
import com.esri.arcgis.server.IServerObjectHelper;
import com.esri.arcgis.system.ILog;
import com.esri.arcgis.system.ServerUtilities;

import static com.esri.arcgis.system.ServerUtilities.*;
import com.esri.arcgis.interop.extn.ServerObjectExtProperties;
import com.esri.arcgis.system.IRESTRequestHandler;

import com.esri.arcgis.server.json.*;

@ArcGISExtension
@ServerObjectExtProperties(
		displayName = "PageLayout SOE", 
		description = "Presents the pagelayout view of layers in associated map service, alongwith with scale bar, north arrow, legend",
		supportsMSD = true
)		
public class PageLayoutSOE implements IServerObjectExtension, IRESTRequestHandler
{
	private static final long serialVersionUID = 1L;
	private IServerObjectHelper soHelper;
	private ILog serverLog;
	private MapServer mapServer;
	
	public PageLayoutSOE() throws Exception
	{
		super();
	}

	/****************************************************************************************************************************
	 * IServerObjectExtension methods: This is a mandatory interface that must be supported by all SOEs. This interface
	 * is used by the Server Object to manage the lifetime of the SOE and includes two methods: init() and shutdown().
	 * The Server Object cocreates the SOE and calls the init() method handing it a back reference to the Server Object
	 * via the Server Object Helper argument. The Server Object Helper implements a weak reference on the Server Object.
	 * The extension can keep a strong reference on the Server Object Helper (for example, in a member variable) but
	 * should not keep a strong reference on the Server Object. The log entries are merely informative and completely
	 * optional.
	 ****************************************************************************************************************************/
	/**
	 * init() is called once, when the instance of the SOE is created.
	 */
	public void init(IServerObjectHelper soh) throws IOException, AutomationException
	{
		/*
		 * An SOE should get the Server Object from the Server Object Helper in order to make any method calls on the
		 * Server Object and release the reference after making the method calls.
		 */
		this.soHelper = soh;
		this.serverLog = getServerLogger();

		// get the Server Object (SO) this SOE is associated with
		this.mapServer = (MapServer) soHelper.getServerObject();
	}

	/**
	 * shutdown() is called once when the Server Object's context is being shut down and is about to go away.
	 */
	public void shutdown() throws IOException, AutomationException
	{
		/*
		 * The SOE should release its reference on the Server Object Helper.
		 */
		this.mapServer = null;
		this.soHelper = null;
		this.serverLog = null;
	}

	/****************************************************************************************************************************
	 * IRESTRequestHandler methods: This interface indicates that SOE supports REST. It exposes two methods: handleRESTRequest()
	 * and getSchema().
	 ****************************************************************************************************************************/
	/**
	 * Handles REST request
	 */	
	public byte[] handleRESTRequest(String capabilities, String resourceName, 
			String operationName, String operationInput, 
			String outputFormat, String requestProperties, 
			String[] responseProperties) throws IOException, AutomationException
	{
		/*
		 * This method handles REST requests by determining whether an operation
		 * or resource has been invoked and then forwards the request to
		 * appropriate methods.
		 */

		try
		{
			// if no operationName is specified send description of specified
			// resource
			if (operationName.length() == 0)
			{
				return getResource(resourceName);
			}
			else
			// invoke REST operation on specified resource
			{
				return invokeRESTOperation(capabilities, resourceName, operationName, operationInput, outputFormat,
						requestProperties, responseProperties);
			}
		}
		catch (Exception e)
		{
			return ServerUtilities.sendError(0, "Exception occurred: " + e.getMessage(), null).getBytes("utf-8");
		}	
	}
	
	/**
	 * Returns schema of the REST resource
	 */
	public String getSchema() throws IOException, AutomationException
	{
		try
		{
			JSONObject _PageLayoutSOE = createResource(
					"PageLayoutSOE",
					"Presents a page layout view of layers in associated map service, alongwith with scale bar, north arrow and legend",
					false);
			JSONArray _PageLayoutSOE_OpArray = new JSONArray();
			_PageLayoutSOE_OpArray.put(createOperation("getPageLayoutBySize", "width, height, dpi", "json, html"));
			_PageLayoutSOE.put("operations", _PageLayoutSOE_OpArray);
			
			JSONArray _PageLayoutSOE_SubResourceArray = new JSONArray();
			JSONObject layout1024x768 = createResource("PageLayout1024x768", "Page layout view of map of dimensions 1024 x 768", false);
			_PageLayoutSOE_SubResourceArray.put(layout1024x768);
			_PageLayoutSOE.put("resources", _PageLayoutSOE_SubResourceArray);			
			
			return _PageLayoutSOE.toString();
		}
		catch (JSONException e)
		{
			e.printStackTrace();
		}
		return null;

	}

	/****************************************************************************************************************************
	 * SOE Util methods.
	 ****************************************************************************************************************************/	
	/**
	 * Invokes specified REST operation on specified REST resource
	 * @param capabilitiesList
	 * @param resourceName
	 * @param operationName
	 * @param operationInput
	 * @param outputFormat
	 * @param requestProperties
	 * @param responseProperties
	 * @return
	 */
	private byte[] invokeRESTOperation(String capabilitiesList, String resourceName, String operationName,
			String operationInput, String outputFormat, String requestProperties, String[] responseProperties) throws Exception
	{
		byte[] operationOutput = null;

		JSONObject operationInputAsJSON = new JSONObject(operationInput);

		//parse request properties and create a map
		java.util.Map<String, String> requestPropertiesMap = new HashMap<String, String>();
		if (requestProperties != null && requestProperties.length() > 0)
		{
			JSONObject requestPropertiesJSON = new JSONObject(requestProperties);
			Iterator<String> jsonKeys = requestPropertiesJSON.keys();
			while (jsonKeys.hasNext())
			{
				String key = jsonKeys.next();
				requestPropertiesMap.put(key, requestPropertiesJSON.getString(key));
			}
		}

		//create a Map to hold response properties
		java.util.Map<String, String> responsePropertiesMap = new HashMap<String, String>();	
		if (resourceName.equalsIgnoreCase("") || resourceName.length() == 0)
		{							
			if (operationName.equalsIgnoreCase("getPageLayoutBySize"))
			{
				operationOutput = generateLayoutAndReturnURL(mapServer, operationInputAsJSON).getBytes("utf-8");
			}
			else
			{
				operationOutput = sendError(0, "Operation " + "\"" + operationName + "\" not supported on resource " + resourceName + ".", 
						new String[]{"No other details", " specified"}).getBytes("utf-8");					
			}				
		}
		else //if non existent sub-resource specified, report error
		{
			operationOutput = sendError(0, "No sub-resource by name \"" + resourceName + "\" found.", new String[]{""}).getBytes("utf-8");	
		}
		
		//convert response properties to String array
		if (!responsePropertiesMap.isEmpty())
		{
			Set<String> keys = responsePropertiesMap.keySet();
			Iterator<String> keysIterator = keys.iterator();
			int i = 0;
			while (keysIterator.hasNext())
			{
				String key = keysIterator.next();
				String value = responsePropertiesMap.get(key);
				responseProperties[i] = key + "=" + value;
				i++;
			}
		}
		
		return operationOutput;	
	}		
	
	/**
	 * Returns description of resource specified by resourceName
	 * @param resourceName
	 * @return byte[]
	 */
	private byte[] getResource(String resourceName)
	{
		try
		{
			if(resourceName.equalsIgnoreCase("") || resourceName.length() == 0)
			{
				//this.serverLog.addMessage(1, 8000, "1");
				return getRootResourceDescription().toString().getBytes();
			}
			if(resourceName.equalsIgnoreCase("PageLayout1024x768"))
			{
				JSONObject json = new JSONObject();
				json.put("height", 1024);
				json.put("width", 768);
				json.put("dpi", 72);
				return generateLayoutAndReturnURL(mapServer, json).getBytes("utf-8");
			}			
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}	
	
	/**
	 * Returns description of the root resource
	 * @return description as a JSONObject
	 */
	private JSONObject getRootResourceDescription()
	{
		try
		{
			JSONObject rootResource = new JSONObject();
			rootResource.put("Name", "PageLayout SOE");
			rootResource.put("Description", "Presents a Map's image in page layout format.");
			
			return rootResource;			
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		return null;
	}	
	
	/**
	 * Generates PageLayout image and returns URL to the image
	 * @param serverObject
	 * @param width
	 * @param height
	 * @param dpi
	 * @return Image URL as String
	 */
	private String generateLayoutAndReturnURL(MapServer serverObject, JSONObject inputParameters)
	{
		try
		{
			int height = inputParameters.getInt("height");
			int width = inputParameters.getInt("width");
			int dpi = inputParameters.getInt("dpi");
			
			if ((height <= 0) || (width <= 0) || (dpi <= 0))
			{
				return sendError(400, "Invalid Input.", null);
			}
			
			// generate page layout.
			PageLayoutImpl impl = new PageLayoutImpl(mapServer, width, height, dpi);
			impl.generatePageLayout();							
			this.serverLog.addMessage(1, 8000, "Layout Output : " + impl.getOutputUrl());

			// set response properties
			
			JSONObject response = new JSONObject();
			response.put("outputUrl", impl.getOutputUrl());
			return response.toString();
		}
		catch (Exception e)
		{
			return sendError(1, e.getMessage(), null);			
		}
	}

	/****************************************************************************************************************************
	 * General Util methods.
	 ****************************************************************************************************************************/
	/**
	 * Creates a REST resource based on specified name, description and collection flag
	 * @param name
	 * @param description
	 * @param isCollection
	 * @return REST resource as a JSONObject
	 */
	private JSONObject createResource(String name, String description, boolean isCollection)
	{
		try
		{
			JSONObject json = new JSONObject();

			if (name.length() > 0 && name != null)
			{
				json.put("name", name);
			}
			else
			{
				throw new Exception("Resource must have a valid name.");
			}

			json.put("description", description);
			json.put("isCollection", isCollection);

			return json;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Creates an operation that can be called on a REST resource, using specified name, parameter list and 
	 * output formats list
	 * @param operationName
	 * @param parameterList
	 * @param supportedOutputFormatsList
	 * @return Operation as a JSONObject
	 */
	private JSONObject createOperation(String operationName, String parameterList, String supportedOutputFormatsList)
	{
		try
		{
			JSONObject operation = new JSONObject();

			if (operationName.length() > 0 && operationName != null)
			{
				operation.put("name", operationName);
			}
			else
			{
				throw new Exception("Operation must have a valid name.");
			}

			// parameters
			if (parameterList.length() > 0 && parameterList != null)
			{
				JSONArray operationParamArray = new JSONArray();
				String[] parameters = parameterList.split(",");
				for (String parameter : parameters)
				{
					operationParamArray.put(parameter.trim());
				}

				operation.put("parameters", operationParamArray);
			}
			else
			{
				throw new Exception(
						"Operation must have parameters. If your operation does not requires params, then please convert it to a sub-resource.");
			}

			// supported Output formats
			if (supportedOutputFormatsList.length() > 0 && supportedOutputFormatsList != null)
			{
				JSONArray outputFormatsArray = new JSONArray();
				String[] outputFormats = supportedOutputFormatsList.split(",");
				for (String outputFormat : outputFormats)
				{
					outputFormatsArray.put(outputFormat.trim());
				}

				operation.put("supportedOutputFormats", outputFormatsArray);
			}
			else
			{
				throw new Exception("Operation must have supported output formats specified");
			}

			return operation;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Returns a detailed error thats constructed based on specified code, message and details array. 
	 * @param code
	 * @param message
	 * @param details
	 * @return error String
	 */
	private String sendError(int code, String message, String[] details)
	{
		/*
		 * An json error is sent back to client in the following structure.{ "error": { "code": 400, "message":
		 * "Cannot perform query. Invalid query parameters.", "details": ["'time' param is invalid"] }}
		 */
		try
		{
			JSONObject errorObject = new JSONObject();

			JSONObject error = new JSONObject();
			error.put("code", code);
			error.put("message", message);

			if (details != null)
			{
				JSONArray detailsArray = new JSONArray();
				for (String detail : details)
				{
					detailsArray.put(detail);
				}

				error.put("details", detailsArray);
			}

			errorObject.put("warning", error);

			return errorObject.toString();
		}
		catch (JSONException e)
		{
			e.printStackTrace();
		}

		return null;
	}

}